#include <stdio.h>
#include <string.h>
#include <errno.h>
#include <gtk/gtk.h>
#include "externs.h"
#include "palette.h"
#include "gkIIfile.h"
#include "misc.h"

int gkII_open_file(image_info* img, char* filename)
{
    FILE* fp;
    imginfo fstr;
    char gkIIid[9];
    guint32* newpal = 0;
    printf("\n\t[debug] Reading gkII file...");
    fp = fopen(filename, "r");
    if (fp == NULL) {
        fprintf(stderr, "Can't open file %s: %s\n", filename,
                strerror(errno));
        return -1;
    }
    fread(&fstr, sizeof(imginfo), 1, fp);
    strncpy(gkIIid, fstr.gkIIid, 8);
    gkIIid[8] = 0;
    if (!strcmp(gkIIid, "gkIIdata") == 0) {
        fprintf(stderr, "File: %s is not a gkII data file.", filename);
        return -2;
    }
    printf("\n\t[debug] Read image info portion...");
    fread(&img->frac_settings, sizeof(fractal_settings), 1, fp);
    printf("\n\t[debug] Read fractal settings...");
    fread(&img->auto_settings, sizeof(auto_layer), 1, fp);
    printf("\n\t[debug] Read auto layer portion...");
    fread(&img->user_settings, sizeof(user_params), 1, fp);
    printf("\n\t[debug] Read user parameters...\n");
    
    if (!(newpal = g_malloc(256* sizeof(guint32))))
        fprintf(stderr, "Out of memory to load colour palette from file %s what a crying shame!", filename);
    else {
        int cip;
        if ((cip = fread(newpal, sizeof(guint32), 256, fp)) < 256)
        {
            fprintf(stderr, "Failed to read entire colour palette from file %s, we got %d instead of 256", filename, cip);
        }
        else {
            g_free(palette);
            palette = newpal;
            printf("\n\t[debug] Read colour palette...\n");
        }
    }
    if (!fread(img->rnd_pal, sizeof(random_palette), 1, fp)) {
        fprintf(stderr, "Failed to read random palette info, could be older gkII-0.4 in which this was not available.");
    }
    fclose(fp);
    set_image_info(img,
        fstr.user_width, fstr.user_height, fstr.aa_factor);
    filestr_to_imagestr(&fstr, img);
    return 0;
}

int gkII_save_file(image_info* img, char* filename)
{
    FILE* fp;
    imginfo fstr;
    strncpy(fstr.gkIIid, "gkIIdata", 8);
    printf("\n\t[debug] Saving gkII file...");
    fp = fopen(filename, "w");
    if (fp == NULL) {
        fprintf(stderr, "Can't open file %s: %s\n", filename,
                strerror(errno));
        return -1;
    }
    imagestr_to_filestr(img, &fstr);
    fwrite(&fstr, sizeof(imginfo), 1, fp);
        printf("\n\t[debug] Written image info portion...");
    fwrite(&img->frac_settings, sizeof(fractal_settings), 1, fp);
        printf("\n\t[debug] Written fractal settings...");
    fwrite(&img->auto_settings, sizeof(auto_layer), 1, fp);
        printf("\n\t[debug] Written auto layer portion...");
    fwrite(&img->user_settings, sizeof(user_params), 1, fp);
        printf("\n\t[debug] Written user parameters...\n");
    fwrite(palette, sizeof(guint32), 256, fp);
        printf("\n\t[debug] Written colour palette...\n");
    fwrite(img->rnd_pal, sizeof(random_palette), 1, fp);
        printf("\n\t[debug] Written random palette info...\n");
    fclose(fp);
    return 0;
}

/* write palette */
/*
    if (pal) {
        png_pal = g_malloc(sizeof(png_color)*pal_indexes);
        for (i=0; i < pal_indexes; i++) {
            png_pal[i].red = RED(palette[i]);
            png_pal[i].green = GREEN(palette[i]);
            png_pal[i].blue = BLUE(palette[i]);
        }
        png_set_PLTE(png_ptr, info_ptr, png_pal, pal_indexes);
    }

    png_write_info(png_ptr, info_ptr);
    if (!pal)
        png_set_filler(png_ptr, 0, PNG_FILLER_AFTER);
*/

/* convert data to palette index format */
/*
    if (pal) {
        int pixels = img->user_width * img->user_height;
        guchar* dst;
        double* src;
        pal_data = g_malloc(pixels);
        dst = pal_data;
        src = img->raw_data;
        for (i=0; i < pixels; i++) {
            if ((*src) >= UINT_MAX)
                *dst = 0;
            else
                *dst = (guint32)(*src) % (pal_indexes-1);
            src++;
            dst++;
        }
    }
*/


gboolean gkII_file_exists(char* filename)
{
    FILE* fp = fopen(filename, "r");
    if (fp == NULL)
        return FALSE;
    fclose(fp);
    return TRUE;
}

void gkII_ask_overwrite(image_info* img, char* filename)
{
    char buf[256];
    GtkWidget* dl;
    GtkWidget* vbox;
    GtkWidget* tmp;

    snprintf(buf, 256, "%s exists, overwrite?", filename);
    dl = gtk_dialog_new();
    gtk_window_set_title(GTK_WINDOW(dl), "File exists!");
    gtk_window_set_resizable(GTK_WINDOW(dl), FALSE);
    gtk_window_set_modal(GTK_WINDOW(dl), TRUE);
    gtk_window_set_position(GTK_WINDOW(dl), GTK_WIN_POS_MOUSE);

    vbox = gtk_vbox_new(FALSE, 0);
    gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dl)->vbox), vbox,
                       TRUE, TRUE, 0);
    gtk_container_set_border_width(GTK_CONTAINER(vbox), 10);
    gtk_widget_show(vbox);

    tmp = gtk_label_new(buf);
    gtk_box_pack_start(GTK_BOX(vbox), tmp, TRUE, TRUE, 0);
    gtk_widget_show(tmp);
    
    tmp = gtk_button_new_with_label("OK");
    gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dl)->action_area),
                       tmp, TRUE, TRUE, 0);
    g_signal_connect(GTK_OBJECT(tmp), "clicked",
                       GTK_SIGNAL_FUNC(gkII_overwrite_ok_cmd),
                       img);
    g_signal_connect_object(GTK_OBJECT(tmp), "clicked",
                              GTK_SIGNAL_FUNC(gtk_widget_destroy),
                              GTK_OBJECT(dl), G_CONNECT_SWAPPED);
    gtk_widget_show(tmp);
    
    tmp = gtk_button_new_with_label("Cancel");
    g_signal_connect_object(GTK_OBJECT(tmp), "clicked",
                              GTK_SIGNAL_FUNC(gtk_widget_destroy),
                              GTK_OBJECT(dl), G_CONNECT_SWAPPED);
    gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dl)->action_area), tmp,
                       TRUE, TRUE, 0);
    gtk_widget_show(tmp);

    gtk_widget_show(dl);
}

void gkII_overwrite_ok_cmd(GtkWidget* w, image_info* img)
{
    gkII_save_file(img,
        (char*)gtk_file_selection_get_filename(
              GTK_FILE_SELECTION(img->filesel) ));
    gtk_widget_destroy(img->filesel);
}

static void gkII_filesave_ok(GtkWidget* w, image_info* img)
{
    char* filename = (char*)gtk_file_selection_get_filename(
                     GTK_FILE_SELECTION(img->filesel));
    if (gkII_file_exists(filename))
        gkII_ask_overwrite(img, filename);
    else {
        gkII_save_file(img, filename);
        gtk_widget_destroy(img->filesel);
    }
}

void do_gkII_save(image_info* img)
{
    if (img->filesel)
        return;
    
    img->filesel = gtk_file_selection_new("Save as gkII File");
    gtk_file_selection_hide_fileop_buttons(
                                GTK_FILE_SELECTION(img->filesel));
    g_signal_connect(GTK_OBJECT(img->filesel), "destroy",
                        GTK_SIGNAL_FUNC(gtk_widget_destroyed),
                        &img->filesel);
    g_signal_connect_object(GTK_OBJECT(
                        GTK_FILE_SELECTION(img->filesel)->cancel_button),
                        "clicked", GTK_SIGNAL_FUNC(gtk_widget_destroy),
                        GTK_OBJECT(img->filesel), G_CONNECT_SWAPPED);
        
    g_signal_connect(GTK_OBJECT
                       (GTK_FILE_SELECTION(img->filesel)->ok_button),
                       "clicked", GTK_SIGNAL_FUNC(gkII_filesave_ok),
                       img);
    gtk_widget_show(img->filesel);
}
