#include <gdk/gdkkeysyms.h>
#include "pal_rot_dlg.h"
#include "main.h"

/* Palette rotation dialogue */

static void pal_rot_destroy(GtkWidget* widget,
                            palette_rotation_dialog* dl);
static void cycle_forward(GtkWidget* widget,
                          palette_rotation_dialog* dl);
static void cycle_backward(GtkWidget* widget,
                           palette_rotation_dialog* dl);
static void step_forward(GtkWidget* widget,
                         palette_rotation_dialog* dl);
static void step_backward(GtkWidget* widget,
                          palette_rotation_dialog* dl);
static void cycle(palette_rotation_dialog* dl, gboolean forward);
static void cycle_stop(palette_rotation_dialog* dl);
static void stop(GtkWidget* widget, palette_rotation_dialog* dl);
static gint key_event(GtkWidget* widget, GdkEventKey* event,
                      palette_rotation_dialog* dl);

/* Palette randomization dialogue */

static void pal_rnd_destroy(GtkWidget* widget,
                            palette_rand_dialog* dl);
static void randomize_palette(GtkWidget* widget,
                            palette_rand_dialog* dl);

gint key_event(GtkWidget* widget, GdkEventKey* event,
               palette_rotation_dialog* dl)
{
    switch (event->keyval) {
    case GDK_plus:
    case GDK_KP_Add:
        step_forward(NULL, NULL);
        break;
    case GDK_minus:
    case GDK_KP_Subtract:
        step_backward(NULL, NULL);
        break;
    case GDK_f:
    case GDK_F:
        cycle(dl, TRUE);
        break;
    case GDK_b:
    case GDK_B:
        cycle(dl, FALSE);
        break;
    case GDK_s:
    case GDK_S:
        cycle_stop(dl);
        break;
    }
    
    return TRUE;
}

void cycle_forward(GtkWidget* widget,
                   palette_rotation_dialog* dl)
{
    cycle(dl, TRUE);
}

void cycle_backward(GtkWidget* widget,
                    palette_rotation_dialog* dl)
{
    cycle(dl, FALSE);
    
}

void cycle(palette_rotation_dialog* dl, gboolean forward)
{
    cycle_stop(dl);

    gtk_widget_set_sensitive(dl->stop, TRUE);
    gtk_widget_set_sensitive(forward ? dl->cycle_b : dl->cycle_f,
                             TRUE);
    gtk_widget_set_sensitive(forward ? dl->cycle_f : dl->cycle_b,
                             FALSE);
    dl->idle_id = g_idle_add((GtkFunction)do_palette_rotation,
                               (gpointer)forward);
}

void cycle_stop(palette_rotation_dialog* dl)
{
    if (dl->idle_id != -1)
        g_source_remove(dl->idle_id);
    gtk_widget_set_sensitive(dl->stop, FALSE);
    gtk_widget_set_sensitive(dl->cycle_f, TRUE);
    gtk_widget_set_sensitive(dl->cycle_b, TRUE);
}

void stop(GtkWidget* widget, palette_rotation_dialog* dl)
{
    cycle_stop(dl);
}

void step_forward(GtkWidget* widget, palette_rotation_dialog* dl)
{
    do_palette_rotation(TRUE);
}

void step_backward(GtkWidget* widget, palette_rotation_dialog* dl)
{
    do_palette_rotation(FALSE);
}

void pal_rot_destroy(GtkWidget* widget,
                     palette_rotation_dialog* dl)
{
    cycle_stop(dl);
    g_free(dl);
}

void palette_rotation_dlg_new(palette_rotation_dialog** ptr,
                              image_info* img)
{
    GtkWidget* hbox;
    palette_rotation_dialog* dl;
    
    dl = g_malloc(sizeof(palette_rotation_dialog));
    dl->img = img;
    dl->idle_id = -1;
    *ptr = dl;
    
    dl->window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
    g_signal_connect(GTK_OBJECT(dl->window), "destroy",
                       GTK_SIGNAL_FUNC(pal_rot_destroy),
                       dl);
    g_signal_connect(GTK_OBJECT(dl->window), "destroy",
                       GTK_SIGNAL_FUNC(gtk_widget_destroyed),
                       ptr);
    gtk_window_set_title(GTK_WINDOW(dl->window), "Palette cycling");
    gtk_window_set_resizable(GTK_WINDOW(dl->window), FALSE);
    g_signal_connect(GTK_OBJECT(dl->window), "key_press_event",
                       GTK_SIGNAL_FUNC(key_event), dl);

    gtk_container_set_border_width(GTK_CONTAINER(dl->window), 10);

    hbox = gtk_hbox_new(FALSE, 5);
    gtk_container_add(GTK_CONTAINER(dl->window), hbox);


    dl->step_b = gtk_button_new_with_label(" < ");
    g_signal_connect(GTK_OBJECT(dl->step_b), "clicked",
                       GTK_SIGNAL_FUNC(step_backward),
                       dl);
    gtk_box_pack_start(GTK_BOX(hbox), dl->step_b, FALSE, FALSE, 0);
    
    dl->cycle_b = gtk_button_new_with_label(" << ");
    g_signal_connect(GTK_OBJECT(dl->cycle_b), "clicked",
                       GTK_SIGNAL_FUNC(cycle_backward),
                       dl);
    gtk_box_pack_start(GTK_BOX(hbox), dl->cycle_b, FALSE, FALSE, 0);

    
    dl->stop = gtk_button_new_with_label(" Stop ");
    g_signal_connect(GTK_OBJECT(dl->stop), "clicked",
                       GTK_SIGNAL_FUNC(stop),
                       dl);
    gtk_box_pack_start(GTK_BOX(hbox), dl->stop, FALSE, FALSE, 4);
    gtk_widget_set_sensitive(dl->stop, FALSE);

    dl->cycle_f = gtk_button_new_with_label(" >> ");
    g_signal_connect(GTK_OBJECT(dl->cycle_f), "clicked",
                       GTK_SIGNAL_FUNC(cycle_forward),
                       dl);
    gtk_box_pack_start(GTK_BOX(hbox), dl->cycle_f, FALSE, FALSE, 0);

    dl->step_f = gtk_button_new_with_label(" > ");
    g_signal_connect(GTK_OBJECT(dl->step_f), "clicked",
                       GTK_SIGNAL_FUNC(step_forward),
                       dl);
    gtk_box_pack_start(GTK_BOX(hbox), dl->step_f, FALSE, FALSE, 0);

    gtk_widget_show_all(dl->window);
}

void palette_rand_dlg_new(palette_rand_dialog** ptr, image_info* img)
{
    GtkWidget* tmp;
    GtkWidget* vbox;
    GtkWidget* r_frame;
    GtkWidget* g_frame;
    GtkWidget* b_frame;
    GtkWidget* r_table;
    GtkWidget* g_table;
    GtkWidget* b_table;

    palette_rand_dialog* dl;

    dl = g_malloc(sizeof(palette_rand_dialog));
    dl->img = img;
    *ptr = dl;

    dl->window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
    g_signal_connect(GTK_OBJECT(dl->window), "destroy",
                       GTK_SIGNAL_FUNC(pal_rnd_destroy),
                       dl);
    g_signal_connect(GTK_OBJECT(dl->window), "destroy",
                       GTK_SIGNAL_FUNC(gtk_widget_destroyed),
                       ptr);
    gtk_window_set_title(GTK_WINDOW(dl->window), "Palette Randomization");
    gtk_window_set_position(GTK_WINDOW(dl->window), GTK_WIN_POS_MOUSE);
    gtk_window_set_resizable(GTK_WINDOW(dl->window), FALSE);
    gtk_container_set_border_width(GTK_CONTAINER(dl->window), 10);

    vbox = gtk_vbox_new(FALSE, 0);
    gtk_container_add(GTK_CONTAINER(dl->window), vbox);
    gtk_widget_show(vbox);

    r_frame = gtk_frame_new("Red");
    g_frame = gtk_frame_new("Green");
    b_frame = gtk_frame_new("Blue");
    gtk_container_set_border_width(GTK_CONTAINER(r_frame),4);
    gtk_container_set_border_width(GTK_CONTAINER(g_frame),4);
    gtk_container_set_border_width(GTK_CONTAINER(b_frame),4);
    gtk_container_add(GTK_CONTAINER(vbox), r_frame);
    gtk_container_add(GTK_CONTAINER(vbox), g_frame);
    gtk_container_add(GTK_CONTAINER(vbox), b_frame);

    r_table = gtk_table_new(2, 4, TRUE);
    tmp = gtk_label_new("Strength:");
    gtk_misc_set_alignment(GTK_MISC(tmp), 0.9, 0.5);
    gtk_table_attach_defaults(GTK_TABLE(r_table), tmp, 0, 1, 0, 1);
    gtk_widget_show(tmp);
    dl->r_strength = gtk_hscale_new_with_range(0.0, 1.0, 0.01);
    gtk_range_set_value(GTK_RANGE(dl->r_strength),
                                    dl->img->rnd_pal->r_strength);
    gtk_table_attach_defaults(GTK_TABLE(r_table), dl->r_strength,
                                                  1, 4, 0, 1);
    gtk_widget_show(dl->r_strength);
    tmp = gtk_label_new("Bands:");
    gtk_misc_set_alignment(GTK_MISC(tmp), 0.9, 0.5);
    gtk_table_attach_defaults(GTK_TABLE(r_table), tmp, 0, 1, 1, 2);
    gtk_widget_show(tmp);
    dl->r_bands = gtk_hscale_new_with_range(0.0, 1.0, 0.01);
    gtk_range_set_value(GTK_RANGE(dl->r_bands), dl->img->rnd_pal->r_bands);
    gtk_table_attach_defaults(GTK_TABLE(r_table), dl->r_bands, 1, 4, 1, 2);
    gtk_widget_show(dl->r_bands);
    gtk_container_add(GTK_CONTAINER(r_frame), r_table);
    gtk_widget_show(r_table);
    gtk_widget_show(r_frame);

    g_table = gtk_table_new(2, 4, TRUE);
    tmp = gtk_label_new("Strength:");
    gtk_misc_set_alignment(GTK_MISC(tmp), 0.9, 0.5);
    gtk_table_attach_defaults(GTK_TABLE(g_table), tmp, 0, 1, 0, 1);
    gtk_widget_show(tmp);
    dl->g_strength = gtk_hscale_new_with_range(0.0, 1.0, 0.01);
    gtk_range_set_value(GTK_RANGE(dl->g_strength),
                                    dl->img->rnd_pal->g_strength);
    gtk_table_attach_defaults(GTK_TABLE(g_table), dl->g_strength,
                                                  1, 4, 0, 1);
    gtk_widget_show(dl->g_strength);
    tmp = gtk_label_new("Bands:");
    gtk_misc_set_alignment(GTK_MISC(tmp), 0.9, 0.5);
    gtk_table_attach_defaults(GTK_TABLE(g_table), tmp, 0, 1, 1, 2);
    gtk_widget_show(tmp);
    dl->g_bands = gtk_hscale_new_with_range(0.0, 1.0, 0.01);
    gtk_range_set_value(GTK_RANGE(dl->g_bands), dl->img->rnd_pal->g_bands);
    gtk_table_attach_defaults(GTK_TABLE(g_table), dl->g_bands, 1, 4, 1, 2);
    gtk_widget_show(dl->g_bands);
    gtk_container_add(GTK_CONTAINER(g_frame), g_table);
    gtk_widget_show(g_table);
    gtk_widget_show(g_frame);

    b_table = gtk_table_new(2, 4, TRUE);
    tmp = gtk_label_new("Strength:");
    gtk_misc_set_alignment(GTK_MISC(tmp), 0.9, 0.5);
    gtk_table_attach_defaults(GTK_TABLE(b_table), tmp, 0, 1, 0, 1);
    gtk_widget_show(tmp);
    dl->b_strength = gtk_hscale_new_with_range(0.0, 1.0, 0.01);
    gtk_range_set_value(GTK_RANGE(dl->b_strength),
                                    dl->img->rnd_pal->b_strength);
    gtk_table_attach_defaults(GTK_TABLE(b_table), dl->b_strength,
                                                  1, 4, 0, 1);
    gtk_widget_show(dl->b_strength);
    tmp = gtk_label_new("Bands:");
    gtk_misc_set_alignment(GTK_MISC(tmp), 0.9, 0.5);
    gtk_table_attach_defaults(GTK_TABLE(b_table), tmp, 0, 1, 1, 2);
    gtk_widget_show(tmp);
    dl->b_bands = gtk_hscale_new_with_range(0.0, 1.0, 0.01);
    gtk_range_set_value(GTK_RANGE(dl->b_bands), dl->img->rnd_pal->b_bands);
    gtk_table_attach_defaults(GTK_TABLE(b_table), dl->b_bands, 1, 4, 1, 2);
    gtk_widget_show(dl->b_bands);
    gtk_container_add(GTK_CONTAINER(b_frame), b_table);
    gtk_widget_show(b_table);
    gtk_widget_show(b_frame);


    dl->apply_button = gtk_button_new_with_label("Apply");
    g_signal_connect(GTK_OBJECT(dl->apply_button), "clicked",
                     GTK_SIGNAL_FUNC(randomize_palette),
                     dl);
    gtk_box_pack_start(GTK_BOX(vbox), dl->apply_button, TRUE, TRUE, 0);
    gtk_widget_show(dl->apply_button);


    gtk_widget_show_all(dl->window);
}

static void randomize_palette(GtkWidget* widget,
                            palette_rand_dialog* dl)
{
    dl->img->rnd_pal->r_strength =
        gtk_range_get_value(GTK_RANGE(dl->r_strength));
    dl->img->rnd_pal->g_strength =
        gtk_range_get_value(GTK_RANGE(dl->g_strength));
    dl->img->rnd_pal->b_strength =
        gtk_range_get_value(GTK_RANGE(dl->b_strength));
    dl->img->rnd_pal->r_bands = gtk_range_get_value(GTK_RANGE(dl->r_bands));
    dl->img->rnd_pal->g_bands = gtk_range_get_value(GTK_RANGE(dl->g_bands));
    dl->img->rnd_pal->b_bands = gtk_range_get_value(GTK_RANGE(dl->b_bands));
    do_palette_randomize(dl->img->rnd_pal);
}

static void pal_rnd_destroy(GtkWidget* widget,
                            palette_rand_dialog* dl)
{
    g_free(dl);
}
